import greenfoot.*;
import java.awt.Color;
public abstract class Button extends Actor
{
    protected String name = "";
    protected int size = 25;
    protected Color fore = Color.BLACK,
        back = Color.CYAN;
    protected GreenfootImage img, over;
    protected boolean mouseOver = false;
    protected World w;
    private int lastX, lastY;

    public void addedToWorld(World w) {
        this.w = w;
        createImages();
        setImage(img);
    }
    
    public void act() {
        int x = getX();
        int y = getY();
        if(Greenfoot.mouseMoved(this)) {
            if(!mouseOver) {
                setImage(over);
                mouseOver = true;
            }
        } else if(Greenfoot.mouseMoved(null)) {
            if(mouseOver) {
                setImage(img);
                mouseOver = false;
            }
        } else if(x != lastX || y != lastY) {
            MouseInfo m = Greenfoot.getMouseInfo();
            if(m != null) {
                int halfW = getImage().getWidth();
                int halfH = getImage().getHeight();
                mouseOver =
                       m.getX() > x - halfW
                    && m.getX() < x + halfW
                    && m.getY() > y - halfH
                    && m.getY() < y + halfH;
                setImage(mouseOver
                    ? over
                    : img);
            }
        }
        lastX = x;
        lastY = y;
        if(Greenfoot.mousePressed(this)) {
            boolean ok = !(w instanceof Menu) || ((Menu)w).hasLoaded();
            if(ok) doAction();
        }
    }
    
    protected void createImages() {
        img = new GreenfootImage(" " + name + " ", size, fore, back);
        Color bg = new Color(back.getRed(), back.getGreen(), back.getBlue(),
            (back.getAlpha() < 128
            ? Math.min(back.getAlpha() + 50, 255)
            : Math.max(back.getAlpha() - 50, 0)));
        over = new GreenfootImage(" " + name + " ", size, fore, bg);
    }
    
    abstract void doAction();
}
