import greenfoot.*;
import java.awt.Color;
public class MCWorld extends World {
    public static boolean SOMETHING_MINING;
    
    public static int X_OFFSET = 50;
    public static int Y_OFFSET = 0;
    
    public static final int BLOCK_WIDTH = 30;
    public static final int BLOCK_HEIGHT = 20;
    public static final int FULL_BLOCK_WIDTH = BLOCK_WIDTH + X_OFFSET*2;
    public static final int FULL_BLOCK_HEIGHT = BLOCK_HEIGHT + 30;
    
    public static final int WIDTH = Block.SIZE*BLOCK_WIDTH;
    public static final int HEIGHT = Block.SIZE*BLOCK_HEIGHT;
    
    private static final int MIN_X_OFFSET = 0;
    private static final int MIN_Y_OFFSET = 0;
    private static final int MAX_X_OFFSET = FULL_BLOCK_WIDTH - BLOCK_WIDTH;
    private static final int MAX_Y_OFFSET = FULL_BLOCK_HEIGHT - BLOCK_HEIGHT;
    
    public static final int cX = WIDTH/2;
    public static final int cY = HEIGHT/2;
    
    private static final int MIN_SCROLL_X = Block.SIZE + Block.HALF_SIZE;
    private static final int MIN_SCROLL_Y = Block.SIZE + Block.HALF_SIZE;
    private static final int MAX_SCROLL_X = WIDTH - MIN_SCROLL_X;
    private static final int MAX_SCROLL_Y = HEIGHT - MIN_SCROLL_Y;
    
    public static Block[][] BLOCKS;
    public static Player PLAYER;
    private BackPanel BACK_PANEL;
    private Inventory inv;
    
    private boolean checkCamMove = true;
    
    public MCWorld() {
        super(WIDTH, HEIGHT, 1, false);
        
        SOMETHING_MINING = false;
        
        X_OFFSET = 50;
        Y_OFFSET = 0;
        
        getBackground().setColor(java.awt.Color.BLACK);
        getBackground().fill();
        
        inv = new Inventory();
        addObject(inv, cX, Inventory.FONT_SIZE/2);
        
        BLOCKS = new Block[FULL_BLOCK_WIDTH][FULL_BLOCK_HEIGHT];
        
        PLAYER = new Player();
        addObject(PLAYER, cX, Block.HALF_SIZE);
        
        genWorld();
        
        BACK_PANEL = new BackPanel();
        addObject(BACK_PANEL, cX, cY);
        addObject(new AlphaOverlay(), cX, cY);
        addObject(new FPS(), 49, 10);
        
        setPaintOrder(FPS.class, Inventory.class, AlphaOverlay.class, Player.class, Block.class, BackPanel.class);
        setActOrder(MCWorld.class, Player.class, Block.class, BackPanel.class, AlphaOverlay.class, Inventory.class);
    }
    
    public void act() {
        checkCamMove = !checkCamMove;
        
        if(Greenfoot.mousePressed(null)) {
            MouseInfo m = Greenfoot.getMouseInfo();
            if(m != null) {
                int mx = m.getX();
                int my = m.getY();
                int mbx = toBlock(mx) + X_OFFSET;
                int mby = toBlock(my) + Y_OFFSET;
                int px = PLAYER.getX();
                int py = PLAYER.getY();
                if(Math.hypot(Math.abs(px - mx), Math.abs(py - my)) < 60
                && BLOCKS[mbx][mby] == null) {
                    Block block = inv.getNewBlock();
                    if(block != null) {
                        if(mbx >= X_OFFSET && mbx < X_OFFSET + BLOCK_WIDTH
                        && mby >= Y_OFFSET && mby < Y_OFFSET + BLOCK_HEIGHT) {
                            addObject(block, toReal(mbx - X_OFFSET), toReal(mby - Y_OFFSET));
                        }
                        BLOCKS[mbx][mby] = block;
                        if(block.isIntersectingPlayer()) {
                            BLOCKS[mbx][mby] = null;
                            inv.addBlock(block.getClass());
                            removeObject(block);
                        }
                    }
                }
            }
        }
        
        if(checkCamMove) {
            if(Greenfoot.isKeyDown("left")
            && PLAYER.getX() <= MAX_SCROLL_X) {
                scrollLeft();
            }
            if(Greenfoot.isKeyDown("right")
            && PLAYER.getX() >= MIN_SCROLL_X) {
                scrollRight();
            }
            if(Greenfoot.isKeyDown("up")
            && PLAYER.getY() <= MAX_SCROLL_Y) {
                scrollUp();
            }
            if(Greenfoot.isKeyDown("down")
            && PLAYER.getY() >= MIN_SCROLL_Y) {
                scrollDown();
            }
        }
    }
    
    private void genWorld() {
        int yVal = 13;
        int minTreeInterval = 6;
        int maxTreeInterval = 20;
        int lastTree = 0;
        for(int x = 0; x < FULL_BLOCK_WIDTH; x++) {
            if(x - lastTree >= minTreeInterval
            && x < FULL_BLOCK_WIDTH - 3
            &&(Greenfoot.getRandomNumber(5) == 0
            || x - lastTree >= maxTreeInterval)) {
                for(int y = yVal - 4; y < yVal; y++) {
                    BLOCKS[x][y] = new Wood();
                }
                for(int y = yVal - 6; y < yVal - 3; y++) {
                    BLOCKS[x-1][y] = new Leaves();
                    BLOCKS[x+1][y] = new Leaves();
                    if(y != yVal - 4) {
                        BLOCKS[x][y] = new Leaves();
                    }
                    if(y != yVal - 6) {
                        BLOCKS[x-2][y] = new Leaves();
                        BLOCKS[x+2][y] = new Leaves();
                    }
                }
                lastTree = x;
            }
            for(int y = yVal; y < FULL_BLOCK_HEIGHT - 1; y++) {
                BLOCKS[x][y] = Greenfoot.getRandomNumber(3) > 0 ? new Dirt() : new Stone();
            }
            BLOCKS[x][FULL_BLOCK_HEIGHT-1] = new Bedrock();
            yVal += Greenfoot.getRandomNumber(3) - 1;
            if(yVal < 8) yVal++;
            if(yVal > 14) yVal--;
        }
        resetBlocks();
    }
    
    public void blockMined(Block block) {
        int x = toBlock(block.getX()) + X_OFFSET;
        int y = toBlock(block.getY()) + Y_OFFSET;
        BLOCKS[x][y] = null;
        inv.addBlock(block.getClass());
        removeObject(block);
    }
    
    public static int toReal(int xy) {
        return xy*Block.SIZE + Block.HALF_SIZE;
    }
    
    public static int toBlock(int xy) {
        return xy/Block.SIZE;
    }
    
    public void scrollLeft() {
        if(X_OFFSET > MIN_X_OFFSET && !SOMETHING_MINING) {
            X_OFFSET--;
            resetBlocks();
            PLAYER.setLocation(PLAYER.getX() + Block.SIZE, PLAYER.getY());
        }
    }
    
    public void scrollRight() {
        if(X_OFFSET < MAX_X_OFFSET && !SOMETHING_MINING) {
            X_OFFSET++;
            resetBlocks();
            PLAYER.setLocation(PLAYER.getX() - Block.SIZE, PLAYER.getY());
        }
    }
    
    public void scrollUp() {
        if(Y_OFFSET > MIN_Y_OFFSET && !SOMETHING_MINING) {
            Y_OFFSET--;
            resetBlocks();
            PLAYER.setLocation(PLAYER.getX(), PLAYER.getY() + Block.SIZE);
            BACK_PANEL.makeBaseImg();
        }
    }
    
    public void scrollDown() {
        if(Y_OFFSET < MAX_Y_OFFSET && !SOMETHING_MINING) {
            Y_OFFSET++;
            resetBlocks();
            PLAYER.setLocation(PLAYER.getX(), PLAYER.getY() - Block.SIZE);
            BACK_PANEL.makeBaseImg();
        }
    }
    
    private void resetBlocks() {
        removeObjects(getObjects(Block.class));
        for(int x = X_OFFSET; x < X_OFFSET + BLOCK_WIDTH; x++)
        for(int y = Y_OFFSET; y < Y_OFFSET + BLOCK_HEIGHT; y++) {
            Block toAdd = BLOCKS[x][y];
            if(toAdd != null) addObject(toAdd, toReal(x - X_OFFSET), toReal(y - Y_OFFSET));
        }
    }
}
